(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
    typeof define === 'function' && define.amd ? define(factory) :
    (global.Angle = factory());
}(this, (function () { 'use strict';

    var version = "1.3.1";

    var classCallCheck = function (instance, Constructor) {
      if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
      }
    };

    var createClass = function () {
      function defineProperties(target, props) {
        for (var i = 0; i < props.length; i++) {
          var descriptor = props[i];
          descriptor.enumerable = descriptor.enumerable || false;
          descriptor.configurable = true;
          if ("value" in descriptor) descriptor.writable = true;
          Object.defineProperty(target, descriptor.key, descriptor);
        }
      }

      return function (Constructor, protoProps, staticProps) {
        if (protoProps) defineProperties(Constructor.prototype, protoProps);
        if (staticProps) defineProperties(Constructor, staticProps);
        return Constructor;
      };
    }();

    var defineProperty = function (obj, key, value) {
      if (key in obj) {
        Object.defineProperty(obj, key, {
          value: value,
          enumerable: true,
          configurable: true,
          writable: true
        });
      } else {
        obj[key] = value;
      }

      return obj;
    };

    /**
     * Angle类(不可变对象结构)
     * 处理角及角单位之间的转换等
     * @author oyzhen <https://git.coding.net/oyzhen/angle.git>
     */

    var Angle = function () {
        /**
         * Angle 类
         */
        var Angle = function () {
            function Angle() {
                var angle = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
                var unit = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Angle.UNIT.RAD;
                classCallCheck(this, Angle);

                Object.defineProperties(this, {
                    angle: {
                        value: angle,
                        enumerable: true
                    },
                    unit: {
                        value: unit,
                        enumerable: true
                    }
                });
            }

            /**
             * @override
             * @returns {string}
             */


            createClass(Angle, [{
                key: 'toString',
                value: function toString() {
                    return this.angle + ' ' + /\(([^)]*)\)/.exec(this.unit.toString())[1];
                }

                /**
                 * 用于JSON.stringify
                 * @override
                 * @returns {string}
                 */

            }, {
                key: 'toJSON',
                value: function toJSON() {
                    return this.toString();
                }

                /**
                 * @override
                 * @returns {number}
                 */

            }, {
                key: 'valueOf',
                value: function valueOf() {
                    return this.rad;
                }

                /**
                 * Object.prototype.toString -> [object Angle]
                 * @returns {string}
                 */

            }, {
                key: 'clone',


                /**
                 * 自拷贝
                 * @returns {Angle}
                 */
                value: function clone() {
                    return new Angle(this.angle, this.unit);
                }

                /**
                 * 获取弧度值
                 * @returns {number}
                 */

            }, {
                key: Symbol.toStringTag,
                get: function get$$1() {
                    return 'Angle';
                }
            }, {
                key: 'rad',
                get: function get$$1() {
                    return toRad(this.angle, this.unit);
                }

                /**
                 * 获取角度值
                 * @returns {number}
                 */

            }, {
                key: 'deg',
                get: function get$$1() {
                    return toDeg(this.angle, this.unit);
                }

                /**
                 * 获取梯度值
                 * @returns {number}
                 */

            }, {
                key: 'grad',
                get: function get$$1() {
                    return toGrad(this.angle, this.unit);
                }

                /**
                 * 获取圈/转数
                 * @returns {number}
                 */

            }, {
                key: 'turn',
                get: function get$$1() {
                    return toTurn(this.angle, this.unit);
                }

                /**
                 * 判断是否是Angle类型
                 * @param {*} angle 待判值
                 */

            }], [{
                key: 'isAngle',
                value: function isAngle(angle) {
                    return angle !== null && angle instanceof Angle;
                }

                /**
                 * 通过弧度值生成Angle对象
                 * @param {number} rad 弧度值
                 * @returns {Angle}
                 */

            }, {
                key: 'fromRad',
                value: function fromRad() {
                    var rad = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

                    return new Angle(rad, Angle.UNIT.RAD);
                }

                /**
                 * 通过角度值生成Angle对象
                 * @param {number} deg 角度值
                 * @returns {Angle}
                 */

            }, {
                key: 'fromDeg',
                value: function fromDeg() {
                    var deg = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

                    return new Angle(deg, Angle.UNIT.DEG);
                }

                /**
                 * 通过梯度值生成Angle对象
                 * @param {number} grad 梯度值
                 * @returns {Angle}
                 */

            }, {
                key: 'fromGrad',
                value: function fromGrad() {
                    var grad = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

                    return new Angle(grad, Angle.UNIT.GRAD);
                }

                /**
                 * 通过圈/转数值生成Angle对象
                 * @param {number} turn 圈数/转数值
                 * @returns {Angle}
                 */

            }, {
                key: 'fromTurn',
                value: function fromTurn() {
                    var turn = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

                    return new Angle(turn, Angle.UNIT.TURN);
                }

                /**
                 * 通过角描述字符串生成Angle对象
                 * @param {string} str 角描述字符串, e.g. 90deg
                 * @returns {Angle}
                 */

            }, {
                key: 'fromString',
                value: function fromString() {
                    var str = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

                    var sep = /(.*?)(grad|deg|rad|turn)$/.exec(str) || [];
                    var angle = parseFloat(sep[1]) || 0;
                    var unit = Angle.UNIT[(sep[2] || 'rad').toUpperCase()];
                    return new Angle(angle, unit);
                }

                /**
                 * 通过角描述字符串生成Angle对象
                 * @param {Angle | string} angle Angle实例或角描述字符串, e.g. '90deg'
                 * @returns {Angle}
                 */

            }, {
                key: 'from',
                value: function from() {
                    var angle = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

                    return Angle.isAngle(angle) ? angle.clone() : Angle.fromString(angle);
                }
            }]);
            return Angle;
        }();

        /**
         * 角单位
         * @prop {Symbol} RAD 弧度
         * @prop {Symbol} DEG 角度
         * @prop {Symbol} GRAD 梯度
         * @prop {Symbol} TURN 圈/转
         */


        Angle.UNIT = Object.freeze({
            RAD: Symbol('rad'),
            DEG: Symbol('deg'),
            GRAD: Symbol('grad'),
            TURN: Symbol('turn')
        });

        /**
         * 版本号
         */
        Angle.version = version;

        return Angle;

        /**
         * 转换为弧度
         * @param {number} angle 角度量值
         * @param {Symbol} type 度量单位
         */
        function toRad(angle, type) {
            var _Angle$UNIT$RAD$Angle;

            return angle && angle * (_Angle$UNIT$RAD$Angle = {}, defineProperty(_Angle$UNIT$RAD$Angle, Angle.UNIT.RAD, 1), defineProperty(_Angle$UNIT$RAD$Angle, Angle.UNIT.DEG, Math.PI / 180), defineProperty(_Angle$UNIT$RAD$Angle, Angle.UNIT.GRAD, Math.PI / 200), defineProperty(_Angle$UNIT$RAD$Angle, Angle.UNIT.TURN, 2 * Math.PI), _Angle$UNIT$RAD$Angle)[type] || angle;
        }

        /**
         * 转换为角度
         * @param {number} angle 角度量值
         * @param {Symbol} type 度量单位
         */
        function toDeg(angle, type) {
            var _Angle$UNIT$RAD$Angle2;

            return angle && angle * (_Angle$UNIT$RAD$Angle2 = {}, defineProperty(_Angle$UNIT$RAD$Angle2, Angle.UNIT.RAD, 180 / Math.PI), defineProperty(_Angle$UNIT$RAD$Angle2, Angle.UNIT.DEG, 1), defineProperty(_Angle$UNIT$RAD$Angle2, Angle.UNIT.GRAD, 0.9), defineProperty(_Angle$UNIT$RAD$Angle2, Angle.UNIT.TURN, 360), _Angle$UNIT$RAD$Angle2)[type] || angle;
        }

        /**
         * 转换为梯度
         * @param {number} angle 角度量值
         * @param {Symbol} type 度量单位
         */
        function toGrad(angle, type) {
            var _Angle$UNIT$RAD$Angle3;

            return angle && angle * (_Angle$UNIT$RAD$Angle3 = {}, defineProperty(_Angle$UNIT$RAD$Angle3, Angle.UNIT.RAD, 200 / Math.PI), defineProperty(_Angle$UNIT$RAD$Angle3, Angle.UNIT.DEG, 10 / 9), defineProperty(_Angle$UNIT$RAD$Angle3, Angle.UNIT.GRAD, 1), defineProperty(_Angle$UNIT$RAD$Angle3, Angle.UNIT.TURN, 400), _Angle$UNIT$RAD$Angle3)[type] || angle;
        }

        /**
         * 转换为圈/转
         * @param {number} angle 角度量值
         * @param {Symbol} type 度量单位
         */
        function toTurn(angle, type) {
            var _Angle$UNIT$RAD$Angle4;

            return angle && angle * (_Angle$UNIT$RAD$Angle4 = {}, defineProperty(_Angle$UNIT$RAD$Angle4, Angle.UNIT.RAD, 0.5 / Math.PI), defineProperty(_Angle$UNIT$RAD$Angle4, Angle.UNIT.DEG, 1 / 360), defineProperty(_Angle$UNIT$RAD$Angle4, Angle.UNIT.GRAD, 0.0025), defineProperty(_Angle$UNIT$RAD$Angle4, Angle.UNIT.TURN, 1), _Angle$UNIT$RAD$Angle4)[type] || angle;
        }
    }();

    return Angle;

})));
